﻿namespace BufferExampleApplication
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Windows;
    using ESRI.ArcGIS.Client;
    using ESRI.ArcGIS.Client.Local;
    using ESRI.ArcGIS.Client.Tasks;

    public class BufferTask
    {
        #region Fields
        private ArcGISLocalFeatureLayer featureLayer;
        private Map map;
        #endregion

        #region Events
        public event EventHandler<StatusUpdateEventArgs> StatusUpdate;
        #endregion

        #region Public methods
        public void Execute(ArcGISLocalFeatureLayer featureLayer, Map map)
        {
            if (featureLayer == null || map == null)
            {
                throw new ArgumentNullException();
            }

            this.featureLayer = featureLayer;
            this.map = map;

            this.StartBufferGeoprocessingService();
        }
        #endregion

        #region Private methods
        private void StartBufferGeoprocessingService()
        {
            string applicationRootFolder = System.IO.Path.GetFullPath(System.IO.Path.Combine(System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), @"..\..\.."));

            string bufferGpk = Path.Combine(applicationRootFolder, @"Data\Buffer.gpk");

            LocalGeoprocessingService.GetServiceAsync(bufferGpk, GPServiceType.SubmitJobWithMapServerResult, (lgs) =>
                {
                    if (lgs.Status == LocalServiceStatus.Running)
                    {
                        this.ExecuteBuffer(lgs);
                    }
                    else
                    {
                        MessageBox.Show("Failed to start GP service");
                    }
                });
        }

        private void ExecuteBuffer(LocalGeoprocessingService lgs)
        {
            if (lgs.Tasks.Count == 0)
            {
                return;
            }

            string uri = lgs.Tasks[0].Url;

            List<GPParameter> parameters = new List<GPParameter>();
            GPFeatureRecordSetLayer param1 = new GPFeatureRecordSetLayer("inputFeature");

            foreach (var g in this.featureLayer.Graphics)
            {
                param1.FeatureSet.Features.Add(g);
            }

            parameters.Add(param1);

            Geoprocessor gp = new Geoprocessor(uri);
            gp.StatusUpdated += this.GeoprocessorStatusUpdated;
            gp.JobCompleted += this.GeoprocessorJobCompleted;
            gp.Failed += this.GeoprocessorFailed;
            gp.SubmitJobAsync(parameters);
        }

        private void GeoprocessorStatusUpdated(object sender, JobInfoEventArgs e)
        {
            var messages = e.JobInfo.Messages;

            this.SendStatusUpdate(messages);
        }

        private void SendStatusUpdate(List<GPMessage> messages)
        {
            if (this.StatusUpdate != null)
            {
                this.StatusUpdate(this, new StatusUpdateEventArgs(messages));
            }
        }

        private void GeoprocessorFailed(object sender, TaskFailedEventArgs e)
        {
            if (e.Error != null)
            {
                var a = e.Error.Data;
                string message = e.Error.Message;
            }
        }

        private void GeoprocessorJobCompleted(object sender, JobInfoEventArgs e)
        {
            if (e.JobInfo != null)
            {
                this.SendStatusUpdate(e.JobInfo.Messages);
            }

            Geoprocessor gp = sender as Geoprocessor;

            if (e.JobInfo.JobStatus == esriJobStatus.esriJobSucceeded)
            {
                gp.GetResultImageLayerCompleted += this.GeoprocessorGetResultImageLayerCompleted;
                gp.GetResultImageLayerAsync(e.JobInfo.JobId, "outputFeatures");
            }
        }

        private void GeoprocessorGetResultImageLayerCompleted(object sender, GetResultImageLayerEventArgs e)
        {
            GPResultImageLayer layer = e.GPResultImageLayer;

            if (layer != null)
            {
                layer.Opacity = 0.5;
                this.map.Layers.Add(layer);
            }
        }
        #endregion
    }
}
