﻿namespace BufferExampleApplication
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using ESRI.ArcGIS.Client.Local;
    using System.Windows;
    using ESRI.ArcGIS.Client.Tasks;
    using System.IO;

    public class CalculateTask
    {
        #region Fields
        private string token = string.Empty;
        private string expression = string.Empty;
        #endregion

        #region Events
        public event EventHandler<StatusUpdateEventArgs> StatusUpdate;
        public event EventHandler<CalculateTaskResultEventArgs> Result;
        #endregion

        #region Public methods
        public void Execute(string token, string expression)
        {
            if (string.IsNullOrEmpty(token) || string.IsNullOrEmpty(expression))
            {
                throw new ArgumentNullException();
            }

            this.token = token;
            this.expression = expression;

            this.StartCalculateGeoprocessingService();
        }
        #endregion

        #region Private methods
        private void StartCalculateGeoprocessingService()
        {
            string applicationRootFolder = System.IO.Path.GetFullPath(System.IO.Path.Combine(System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), @"..\..\.."));

            string calculateGpk = Path.Combine(applicationRootFolder, @"Data\Calculator.gpk");

            LocalGeoprocessingService.GetServiceAsync(calculateGpk, GPServiceType.SubmitJob, (lgs) =>
                {
                    if (lgs.Status == LocalServiceStatus.Running)
                    {
                        this.ExecuteCalculate(lgs);
                    }
                    else
                    {
                        MessageBox.Show("Failed to start GP service");
                    }
                });
        }

        private void ExecuteCalculate(LocalGeoprocessingService lgs)
        {
            if (lgs.Tasks.Count == 0)
            {
                return;
            }

            string uri = lgs.Tasks[0].Url;

            List<GPParameter> parameters = new List<GPParameter>();

            parameters.Add(new GPString("Expression", this.expression));

            Geoprocessor gp = new Geoprocessor(uri);
            gp.Token = this.token;
            gp.StatusUpdated += this.GeoprocessorStatusUpdated;
            gp.JobCompleted += this.GeoprocessorJobCompleted;
            gp.Failed += this.GeoprocessorFailed;
            gp.SubmitJobAsync(parameters);
        }

        private void GeoprocessorStatusUpdated(object sender, JobInfoEventArgs e)
        {
            var messages = e.JobInfo.Messages;

            this.SendStatusUpdate(messages);
        }

        private void SendStatusUpdate(List<GPMessage> messages)
        {
            if (this.StatusUpdate != null)
            {
                this.StatusUpdate(this, new StatusUpdateEventArgs(messages));
            }
        }

        private void GeoprocessorFailed(object sender, TaskFailedEventArgs e)
        {
            if (e.Error != null)
            {
                var a = e.Error.Data;
                string message = e.Error.Message;
            }
        }

        private void GeoprocessorJobCompleted(object sender, JobInfoEventArgs e)
        {
            if (e.JobInfo != null)
            {
                this.SendStatusUpdate(e.JobInfo.Messages);
            }

            Geoprocessor gp = sender as Geoprocessor;

            if (e.JobInfo.JobStatus == esriJobStatus.esriJobSucceeded)
            {
                gp.GetResultDataCompleted += this.GeoprocessorGetResultDataCompleted;
                gp.GetResultDataAsync(e.JobInfo.JobId, "outputValue");
            }
        }

        private void GeoprocessorGetResultDataCompleted(object sender, GPParameterEventArgs e)
        {
            Geoprocessor gp = sender as Geoprocessor;

            GPString outputValueParameter = e.Parameter as GPString;

            if (gp != null && outputValueParameter != null)
            {
                string value = outputValueParameter.Value;

                if (this.Result != null)
                {
                    this.Result(this, new CalculateTaskResultEventArgs(gp.Token, value));
                }
            }
        }
        #endregion
    }
}
