﻿namespace BufferExampleApplication
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Windows;
    using ESRI.ArcGIS.Client;
    using ESRI.ArcGIS.Client.Local;
    using ESRI.ArcGIS.Client.Tasks;
    using System.Threading;
    using System.Diagnostics;

    /// <summary>
    /// MainWindow class.
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Fields
        /// <summary>
        /// Used to generate unique token for Calculate Task.
        /// </summary>
        private static int executeCounter = 1;
        
        /// <summary>
        /// A global reference to the layer that we will be adding.
        /// </summary>
        private ArcGISLocalFeatureLayer featureLayer;
        #endregion

        #region Constructor
        public MainWindow()
        {
            InitializeComponent();

            // Set the ArcGIS Runtime license by providing the license string.
            LocalServer.SetLicense(string.Empty);

            // Confirm the license is valid prior to using ArcGIS Runtime functionality. 
            // Once the LocalServer has started, the LicenseStatus property will be populated.
            LocalServer.InitializeAsync(() =>
            {
                if (LocalServer.LicenseStatus != LicenseStatus.Valid)
                {
                    MessageBox.Show("This ArcGIS Runtime license is not valid");
                }
            });

            LocalServer.InitializeAsync(() =>
            {
                if (LocalServer.LicenseStatus != LicenseStatus.Valid)
                {
                    MessageBox.Show("The ArcGIS Runtime license is invalid, please contact your administrator.");
                }
                else
                {
                    this.AddParkLayer();
                }
            });
        }
        #endregion

        #region Private methods
        /// <summary>
        /// Add Acadia NPS Park layer to map.
        /// </summary>
        private void AddParkLayer()
        {
            string applicationRootFolder = System.IO.Path.GetFullPath(System.IO.Path.Combine(System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), @"..\..\.."));

            string map = Path.Combine(applicationRootFolder, @"Data\acadia.mpk");

            this.featureLayer = new ArcGISLocalFeatureLayer(map, "Acadia");
            this.featureLayer.ID = "Acadia";
            this.featureLayer.InitializationFailed += this.InitializationFailed;
            this.mapControl.Layers.Add(this.featureLayer);
        }

        /// <summary>
        /// Updates the status textbox.
        /// </summary>
        /// <param name="messages">The messages to write to the status textbox.</param>
        private void UpdateStatus(List<GPMessage> messages)
        {
            this.statusBox.Text = string.Empty;

            foreach (GPMessage message in messages)
            {
                this.statusBox.Text += message.Description + Environment.NewLine;
            }
        }

        /// <summary>
        /// Displayed when a layer failed to initialize.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void InitializationFailed(object sender, System.EventArgs e)
        {
            ArcGISLocalFeatureLayer localFeatureLayer = sender as ArcGISLocalFeatureLayer;
            ArcGISTiledMapServiceLayer tiledLayer = sender as ArcGISTiledMapServiceLayer;

            if (localFeatureLayer != null)
            {
                /* error while loading map */
                MessageBox.Show("Failed to initialize map: " + localFeatureLayer.Url);
            }
            else if (tiledLayer != null)
            {
                /* error while loading map */
                MessageBox.Show("Failed to initialize map: " + tiledLayer.Url);
            }
            else
            {
                /* error while loading map */
                MessageBox.Show("Failed to initialize map.");
            }
        }

        /// <summary>
        /// Handles button click event. Executes BufferTask.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExecuteBufferClick(object sender, RoutedEventArgs e)
        {
            if (this.featureLayer != null && this.featureLayer.IsInitialized)
            {
                BufferTask bt = new BufferTask();
                bt.StatusUpdate += this.HandleStatusUpdateEvent;
                bt.Execute(this.featureLayer, this.mapControl);
            }
        }

        /// <summary>
        /// Handles StatusUpdate event of Tasks.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void HandleStatusUpdateEvent(object sender, StatusUpdateEventArgs e)
        {
            if (e.Messages != null)
            {
                this.UpdateStatus(e.Messages);
            }
        }

        /// <summary>
        /// Handles button click event. Executes BufferTask.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExecuteCalculateClick(object sender, RoutedEventArgs e)
        {
            string expression = this.expressionBox.Text;

            if (!string.IsNullOrEmpty(expression))
            {
                CalculateTask ct = new CalculateTask();
                ct.StatusUpdate += this.HandleStatusUpdateEvent;
                ct.Result += this.HandleCalculateTaskResult;
                ct.Execute(executeCounter.ToString(), expression);
                executeCounter++;
            }
        }

        /// <summary>
        /// Handles Result event of Calculate Task.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void HandleCalculateTaskResult(object sender, CalculateTaskResultEventArgs e)
        {
            string token = e.Token;
            string value = e.Result;

            if (!string.IsNullOrEmpty(value))
            {
                this.resultsBox.Text += string.Format("task #{0}: {1}{2}", token, value, Environment.NewLine);
            }
        }

        /// <summary>
        /// Opens the ArcGIS Runtime Webservice webpage.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItem1Click(object sender, RoutedEventArgs e)
        {
            if (LocalServer.IsRunning)
            {
                Thread t = new Thread(new ThreadStart(() =>
                {
                    Process.Start(LocalServer.Url);
                }));

                t.Start();
            }
        }

        /// <summary>
        /// Opens the ArcGIS Runtime Webservice admin webpage.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItem2Click(object sender, RoutedEventArgs e)
        {
            if (LocalServer.IsRunning)
            {
                Thread t = new Thread(new ThreadStart(() =>
                {
                    string uri = LocalServer.Url.Replace("services", "admin");

                    Process.Start(uri);
                }));

                t.Start();
            }
        }
        #endregion
    }
}
