﻿using System.Windows;
using ESRI.ArcGIS.Client.Local;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Tasks;
using System.IO;
using System.Windows.Media;
using ESRI.ArcGIS.Client.Symbols;

namespace DynamicLayersExample
{
    public partial class MainWindow : Window
    {
        private string[] layers = new string[] { "acadia", "acadiaBuffer" };
        private static string WorkspaceId = "exampleGdb";
        private bool switcher = false;

        public MainWindow()
        {
            InitializeComponent();

            string applicationFolder = System.IO.Path.GetFullPath(System.IO.Path.Combine(System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), @"..\.."));
            string dataFolder = Path.Combine(applicationFolder, "Data");
            string fileGdb = Path.Combine(dataFolder, "example.gdb");
            string mapPackage = Path.Combine(dataFolder, "acadia.mpk");

            if (!File.Exists(mapPackage) || !Directory.Exists(fileGdb))
            {
                throw new FileNotFoundException();
            }

            LocalMapService localMapService = new LocalMapService(mapPackage);
            localMapService.EnableDynamicLayers = true;

            WorkspaceInfo workspaceInfo = new WorkspaceInfo(WorkspaceId, WorkspaceFactoryType.FileGDB, string.Format("DATABASE={0}", fileGdb));

            localMapService.DynamicWorkspaces.Add(workspaceInfo);

            localMapService.StartAsync((callback) =>
                {
                    if (callback.Status == LocalServiceStatus.Running)
                    {
                        this.ShowMap(localMapService);
                    }
                });
        }

        private void ShowMap(LocalMapService localMapService)
        {
            ArcGISLocalDynamicMapServiceLayer dynamiclayer = new ArcGISLocalDynamicMapServiceLayer(localMapService);
            dynamiclayer.ImageFormat = ArcGISDynamicMapServiceLayer.RestImageFormat.PNG32;
            dynamiclayer.InitializationFailed += this.InitializationFailed;

            this.MyMap.Layers.Add(dynamiclayer);

            this.switchBtn.IsEnabled = true;
        }

        /// <summary>
        /// Displayed when a layer failed to initialize.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void InitializationFailed(object sender, System.EventArgs e)
        {
            ArcGISLocalFeatureLayer localFeatureLayer = sender as ArcGISLocalFeatureLayer;
            ArcGISTiledMapServiceLayer tiledLayer = sender as ArcGISTiledMapServiceLayer;

            if (localFeatureLayer != null)
            {
                /* error while loading map */
                MessageBox.Show("Failed to initialize map: " + localFeatureLayer.Url);
            }
            else if (tiledLayer != null)
            {
                /* error while loading map */
                MessageBox.Show("Failed to initialize map: " + tiledLayer.Url);
            }
            else
            {
                /* error while loading map */
                MessageBox.Show("Failed to initialize map.");
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            var dynamicLayer = this.MyMap.Layers[1] as ArcGISLocalDynamicMapServiceLayer;

            // The following code will get the dynamic layer info collection from the layer (service)
            // and update the first item with a new DynamicLayerInfo object to change the data 
            // source of an existing feature layer within the service. Alternatively you can create a 
            // brand new DynamicLayerInfoCollection and begin adding new DynamicLayerInfos to this 
            // collection. Doing this will effectively remove the existing layers and display only the new 
            // feature layers defined. 
            if (dynamicLayer != null)
            {
                // options will be null... bug?!
                var options = dynamicLayer.LayerDrawingOptions;

                string layer = string.Empty;

                if (switcher)
                {
                    layer = this.layers[0];
                }else {
                    layer = this.layers[1];
                }

                switcher = !switcher;
                
                var layers = dynamicLayer.CreateDynamicLayerInfosFromLayerInfos();

                layers[0] = new DynamicLayerInfo()
                {
                    ID = 0,
                    Source = new LayerDataSource()
                    {
                        DataSource = new TableDataSource()
                        {
                            DataSourceName = layer,
                            WorkspaceID = WorkspaceId,
                        }
                    }
                };

                dynamicLayer.DynamicLayerInfos = layers;

                // this won't work because options is null; let's define our own renderer.
                ////dynamicLayer.LayerDrawingOptions = options;

                LayerDrawingOptions layerDrawOpt = new LayerDrawingOptions();
                layerDrawOpt.Renderer = new SimpleRenderer() { Symbol = GetSimpleFillSymbol(Colors.Orange, Colors.Black, 1) };
                dynamicLayer.LayerDrawingOptions = new LayerDrawingOptionsCollection() { layerDrawOpt };

                dynamicLayer.Refresh();

                ////this.MyLegend.Refresh();
            }
        }

        private SimpleFillSymbol GetSimpleFillSymbol(Color fillColor, Color borderColor, double borderThickness)
        {
            SimpleFillSymbol fillSymbol = new SimpleFillSymbol()
            {
                BorderBrush = new SolidColorBrush(borderColor) { Opacity = 0.75 },
                BorderThickness = borderThickness,
                Fill = new SolidColorBrush(fillColor)
            };
            return fillSymbol;
        }
    }
}
